#define __SINST__		// g++ header bug
#include "AST.h"
#include "CfCode.h"
#include "compiler.h"
#include "config.h"
#include "ctBox.h"
#include "decls.h"
#include "errors.h"
#include "string-erase.h"


#define BASE_TO_HDR(basevar, base, header, end_str) if ((basevar) == base) return string(header) + end_str


#ifdef USE_NEW_ARRAYS
static bool prefix(const string &head, const string &full, string::size_type pos = 0)
{
    return pos >= head.length()
	|| (pos < full.length() &&
	    head[ pos ] == full[ pos ] &&
	    prefix(head, full, pos + 1));
}
#endif


/*
 * This is silly.  We shouldn't have to decompose and recompose the
 * name.  We should have a more structured interface to start with
 * instead of using fullName().
 */
static string getRoot(ClassDecl *c)
{
    string baseName = c->fullName('_');

#ifdef USE_NEW_ARRAYS
    if (prefix( "ti_internal_tiJArray<", baseName )) {
	const size_t baseLength = baseName.length();
	string result;
	result.reserve( baseLength );
	for (size_t position = 0; position < baseLength; ++position)
	    switch (baseName[ position ]) {
	    case '<':
	    case ',':
		result += '_';
		break;
	    case '>':
	    case ' ':
		break;
	    default:
		result += baseName[ position ];
	    }
	return result;
    }
#endif
    
    if (baseName[ baseName.length() - 1 ] == '>') {
        char arity = baseName[ baseName.length() - 2 ];
	string::size_type pos;
	if ((pos = baseName.find('<')) != string::npos)
	    stringErase( baseName, pos );
	BASE_TO_HDR(baseName, "ti_internal_tiJArray", "JAVA_ARRAY", "");
	else BASE_TO_HDR(baseName, "ti_internal_tiArray", "TI_ARRAY", "");
	else BASE_TO_HDR(baseName, "ti_internal_tiArrayL", "TI_ARRAY", "");
	else BASE_TO_HDR(baseName, "ti_internal_tiRectDomain", "ti_domains_tiRectDomain", arity);
	else BASE_TO_HDR(baseName, "ti_internal_tiDomain", "ti_domains_tiDomain", arity);
	else BASE_TO_HDR(baseName, "ti_internal_tiPoint", "ti_domains_tiPoint", arity);
	else
	    Error( "internal error: need special case header handler for "
		   + c->errorName() );
    }
    return baseName;
}


const string ClassDecl::cDescriptorName()
{ 
    return cType().name + "_descriptor";
}


const string ClassDecl::cIntfMTName()
{
    return cType().name + "_intf_mt";
}

const string ClassDecl::cInitFieldsName()
{
    return cType().name + "_init_instance_fields";
}
    

const string ClassDecl::cNativeName()
{
    return "native/" + fullName('_') + ".c";
}


const string ClassDecl::cNullifyLocalName()
{
  return cType().name + "_nullify_local_fields";
}


void ClassDecl::includeSelf( CfCode &os )
{
  if (asType()->isPointType())
    os.include( "domains" );
  
  else if (asType()->isTitaniumArrayType())
    {
      assert( !asType()->isLocal() );
      os.include( asType()->cType() );
    }

  else
    os.include( cType() );
}


void ClassDecl::includeSupers( CfCode &os )
{
    Decl * const super = (this == ObjectDecl ? 0 : superClass());
    if (super) super->includeSelf( os );
    
    foriter (interface, elements( interfaces() ), ListIterator< Decl * >)
	(*interface)->includeSelf( os );
}


const char *ClassDecl::emitInterfaceList( ostream &os )
{
    if (interfaces()) {
  	os << "static const interface_header *interfaces[] = {\n";
	foriter (interface, elements( interfaces() ), ListIterator< Decl * >) {
	    ClassDecl &decl = static_cast< ClassDecl & >(**interface);
	    assert( decl.category() == Decl::Interface );
	    os << "  &" << decl.cDescriptorName() << ",\n";
	}
	os << "  0\n};\n\n";
	return "interfaces";
    } else
	return "0";
}


void ClassDecl::includeRequirements( CfCode &os )
{
    DeclSet &classes = requires();
    
    for (DeclSet::iterator dependency = classes.begin();
	 dependency != classes.end(); ++dependency)
	(*dependency)->includeSelf( os );

    os << '\n';
}


// stubs

const string Decl::cIntfMTName()
{
    invalidOperation("cIntfMTName");
    return "";
}


const string Decl::cNativeName()
{
    invalidOperation("cNativeName");
    return "";
}


void Decl::includeSupers(CfCode &)
{
    invalidOperation("includeSupers");
}


void Decl::includeSelf(CfCode &)
{
    invalidOperation("includeSelf");
}
