#include "AST.h"
#include "CodeContext.h"
#include "CtReference.h"
#include "FieldDecl.h"
#include "code.h"
#include "ctBox.h"
#include "decls.h"
#include "jni-bridge.h"
#include "lgMacro.h"


const string TypeFieldAccessNode::emitExpression2( CodeContext & )
{
    Decl *d = simpName()->decl();
    TreeNode *t = ftype()->decl()->asType();

    // methods should go through emitMethodCall
    assert( !(d->category() & Decl::Method) );
    assert( d->modifiers() & Common::Static );

    // side-effect free
    return "(STATIC_REF(" + cFieldName(d) + "))";
}


const string ObjectFieldAccessNode::emitExpression2( CodeContext &context )
{
    Decl *d = simpName()->decl();
    TypeNode &objectType = *accessedObjectType();

    // methods should go through emitMethodCall
    assert( !(d->category() & Decl::Method) );

    if (d->modifiers() & Static)
	return "(STATIC_REF(" + cFieldName(d) + "))";
    
    string result;
    
    if ((objectType.isPointType()
	 || objectType.isRectDomainType()
	 || objectType.isTitaniumArrayType()))
        result = '(' + object()->defineTemporary( context ) + ")." +
		    *(d->name());
    
    else if (objectType.isImmutable())
        result = '(' + object()->defineTemporary( context ) + "." +
		    cFieldName(d) + ')';
    
    else {
	const bool objectLocal = objectType.isLocal();
	const string address = getLvalue( context );
	
	const TypeNode &fieldType = *decl()->type();
	const string fieldValue = declareTemporary( context, fieldType.cType() );

	context << lgMacro( "DEREF", objectLocal ) << '('
		<< fieldValue << ", " << address << ");"
		<< endCline;

	if (!objectLocal && fieldType.isLocal())
	  {
	    // when accessing object.field, with a global object and a
	    // local field, the resulting field value must be widened
	    // from local to global
	    const string proc = "TO_PROC(" + address + ')';
	    result = fieldType.emitGlobalize( context, proc, fieldValue );
	  }
	else
	  result = fieldValue;
    }

    return result;
}

const bool TypeFieldAccessNode::isLocalLvalue()
{
  return true;
}

const string TypeFieldAccessNode::getLvalue( CodeContext & )
{
    Decl *d = simpName()->decl();
    TreeNode *t = ftype()->decl()->asType();

    // methods should go through emitMethodCall
    assert( !(d->category() & Decl::Method) );
    assert( d->modifiers() & Common::Static );

    // side-effect free
    string result;
    return string("(& STATIC_REF(") + cFieldName(d) + string("))");
}

const bool ObjectFieldAccessNode::isLocalLvalue()
{
  return accessedObjectType()->isLocal()
    || accessedObjectType()->isImmutable()
    || simpName()->decl()->modifiers() & Static;
}

const string ObjectFieldAccessNode::getLvalue( CodeContext &context )
{
  FieldDecl &d = static_cast< FieldDecl & >(*decl());

    // methods should go through emitMethodCall
  assert( d.category() == Decl::Field );

  if (d.modifiers() & Static)
    {
      object()->emitExpression(context);
      return string("(& STATIC_REF(") + cFieldName(&d) + string("))");
    }

  TypeNode &objectType = *accessedObjectType();

  assert(!objectType.isPointType());
  assert(!objectType.isRectDomainType());
  assert(!objectType.isTitaniumArrayType());

  // need to know the interior layout of the containing instance
  context.depend( objectType.decl()->cType() );

  string fieldName;	
  if (objectType.isJavaArrayType()) {
    assert( *d.name() == "length" );
    fieldName = "header.length";
  } else
    fieldName = cFieldName(&d);
	
  if (objectType.isImmutable())
    // assignments to immutable objects only occur in constructors, and
    // to the fields of the object being constructed
    return "&(me." + fieldName + ')';
  else {
    const string objectValue = object()->defineTemporary( context );
    const bool objectLocal = objectType.isLocal();
    const CtReference &addrType = ctBox( d.cType(), objectLocal );
    const string addrValue = declareTemporary( context, addrType );
    
    context << lgMacro( "FIELD_ADDR", objectLocal )
	    << '(' << addrValue << ", " << objectValue
	    << ", " << fieldName << ");" << endCline;

    return addrValue;
  }
}
