#include <stdlib.h>
#include <sys/time.h>
#include <pwd.h>
#include <unistd.h>

#include <class_header.h>
#include <java_string.h>
#include <native-stubs.h>
#include "runtime-options.h"
#include "layout!GP_JA_jbyte.h"
#include "layout!GP_LP_cl_java_lang_Object_desc.h"
#include "layout!GP_class_header.h"
#include "layout!GP_jbyte.h"
#include "layout!GP_jint.h"
#include "layout!JA_jbyte.h"

extern char *RUNTIME_MODEL;

static void checkIsArray( GP_cl_java_lang_Object object )
{
    typedef GP_type( TypeCategory ) GP_TypeCategory;

    cl_java_lang_Object_desc *info;
    GP_class_header header;
    GP_TypeCategory categoryField;
    TypeCategory category;

    CLASS_INFO_GLOBAL( info, cl_java_lang_Object_desc, object );
    TO_GLOBAL( header, TO_PROC( object ), (class_header *) info );
    FIELD_ADDR_GLOBAL( categoryField, header, category );
    DEREF_GLOBAL( category, categoryField );

    assert( category == JavaArray );
}
  

void cl_java_lang_System_mt_arraycopy_GP_cl_java_lang_Object_jint_GP_cl_java_lang_Object_jint_jint(
    GP_cl_java_lang_Object src,
    jint src_pos,
    GP_cl_java_lang_Object dst,
    jint dst_pos,
    jint length)
{
    GP_JA_jbyte src_array, dst_array;
    jint src_size, dst_size;
    GP_jbyte src_data, dst_data;
    checkIsArray( src );
    checkIsArray( dst );

    src_array = * (GP_JA_jbyte *) &src;
    dst_array = * (GP_JA_jbyte *) &dst;

    JAVA_ARRAY_SIZE_GLOBAL( src_size, src_array );
    JAVA_ARRAY_SIZE_GLOBAL( dst_size, dst_array );
    assert( src_size == dst_size );

    JAVA_ARRAY_CHECK_GLOBAL( src_array, src_pos, length );
    JAVA_ARRAY_CHECK_GLOBAL( dst_array, dst_pos, length );

    /* We would like to use JAVA_ARRAY_ADDR_GLOBAL for the
       indexing computations below, but we cannot.  The arrays
       might not really be arrays of bytes, and so the bounds
       check inside JAVA_ARRAY_ADDR_GLOBAL will fail
       inappropriately.
       */
    
    FIELD_ADDR_GLOBAL( src_data, src_array, data[0] );
    INDEX_GLOBAL( src_data, src_data, src_pos * src_size );
    
    FIELD_ADDR_GLOBAL( dst_data, dst_array, data[0] );
    INDEX_GLOBAL( dst_data, dst_data, dst_pos * dst_size );

    {
	int copy_sz = src_size * length;
	VARARRAY_DECLARE(jbyte, tmp_buf, copy_sz);
	VARARRAY_CREATE(jbyte, tmp_buf, copy_sz);
	global_to_local_copy(src_data, tmp_buf, copy_sz);
	local_to_global_copy(tmp_buf, dst_data, copy_sz);
	VARARRAY_DESTROY(tmp_buf);
    }
}


jlong cl_java_lang_System_mt_currentTimeMillis()
{
    struct timeval now;
    assert(!gettimeofday(&now, 0));
    
    /* We could round the secs, but it does not seem worth it. */
    return now.tv_sec * (jlong) 1000 + now.tv_usec / 1000;
}

static void addProperty( GP_cl_java_util_Properties props, char keyC[], char valueC[] )
{
  if (valueC)
    {
      const LP_JString key = java_string_build_8( keyC );
      const LP_JString value = java_string_build_8( valueC );
      cl_ti_lang_NativeUtils_mt_addProperty_GP_cl_java_util_Properties_LP_cl_java_lang_String_LP_cl_java_lang_String( props, key, value );
    }
}    
    

GP_cl_java_util_Properties
cl_java_lang_System_mt_initProperties_GP_cl_java_util_Properties( GP_cl_java_util_Properties props )
{
  const struct passwd * const user = getpwuid( getuid() );

  addProperty( props, "java.version",		"0" );
  addProperty( props, "java.vendor",		"Titanium Project" );
  addProperty( props, "java.vendor.url",	"http://HTTP.CS.Berkeley.EDU/Research/Projects/titanium/" );
  addProperty( props, "java.home",		"/" );
  addProperty( props, "java.class.version",	"0" );
  addProperty( props, "java.class.path",	"/" );
  addProperty( props, "os.name",		"Titanium" );
  addProperty( props, "os.arch",		"Titanium" );
  addProperty( props, "os.version",		"0" );
  addProperty( props, "file.separator",	"/" );
  addProperty( props, "path.separator",	":" );
  addProperty( props, "line.separator",	"\n" );
  /* Possible values: split-c, smp, uniproc, pthread */
  addProperty( props, "runtime.model", RUNTIME_MODEL);
  if (strcmp(RUNTIME_MODEL, "split-c") == 0) {
    addProperty( props, "runtime.distributed", "true");
  }
  else {
    addProperty( props, "runtime.distributed", "false");
  }

  if (user) {
    addProperty( props, "user.name",		user->pw_name );
    addProperty( props, "user.home",		user->pw_dir );
  }
    
  addProperty( props, "user.dir",		getcwd( 0, 0 ) );

  return props;
}
