default : all


########################################################################
#
#  Timestamps to record incremental progress
#

stamp_tc    := $(generate_dir)/stamp-tc
stamp_cache := $(generate_dir)/stamp-cache


########################################################################
#
#  What's what, and what depends upon what
#
#  The order of the following lines is quite particular.
#

depends_mk := $(generate_dir)/depends.mk

include $(config_mk_dir)/config.mk
include $(mk_dir)/$(context).mk
include $(mk_dir)/$(strategy).mk

objects := $(sources:.c=.o)


########################################################################
#
#  Various and sundry supporting makefile fragments
#

include $(mk_dir)/$(goal).mk
include $(backend_mk_dir)/$(backend).mk


########################################################################
#
#  Prebuilt Titanium class library
#

ifneq "$(prebuilt_tlib)" ""
tlib_includes := -I$(tlib_include_dir)
tlib_ld_libs  := -ltlib-$(backend)-$(prebuilt_tlib)
endif


########################################################################
#
#  Command options for Titanium compiler, C compiler, and linker
#
#  The "clean" versions avoid any mention of $(generate_dir), and
#  therefore are suitable for validating cache entries.
#

clean_tc_command := CLASSLIB=$(classlib) $(tc) $(tc_flags)
tc_command := $(clean_tc_command) -outdir $(generate_dir)

clean_includes := $(tlib_includes) $(context_includes) $(backend_includes)
clean_cc_flags := $(cc_flags) $(backend_defines) $(clean_includes)
clean_cc_command := $(cc) $(clean_cc_flags)

unclean_includes := -I$(generate_dir)

all_includes := $(clean_includes) $(unclean_includes)
all_cc_flags := $(clean_cc_flags) $(unclean_includes)
cc_command   := $(clean_cc_command) $(unclean_includes)

all_ld_flags := $(ld_flags) $(backend_ld_flags) $(context_ld_flags)
all_ld_libs  := $(ld_libs) $(tlib_ld_libs) $(backend_ld_libs) -lm


########################################################################
#
#  Provide the requested level of feedback
#

ifeq "$(silent)" "0"
message := @echo
pipe_demangler := | $(demangler)
else
message := @:
endif

message_link = $(message) Linking $@ ...


########################################################################
#
#  Targets
#

makefile := $(generate_dir)/GNUmakefile
intermediate1 := $(generate_dir)/stamp-*
intermediate2 := $(generate_dir)/*.mk
intermediate3 := $(generate_dir)/*.c
intermediate4 := $(generate_dir)/*.h
intermediate5 := $(generate_dir)/*.o

$(stamp_tc) : $(makefile)
	$(message) Removing old intermediate files ...
	rm -f $(intermediate1)
	rm -f $(intermediate2)
	rm -f $(intermediate3)
	rm -f $(intermediate4)
	rm -f $(intermediate5)
	[ -d $(generate_dir) ] || mkdir -p $(generate_dir)
	$(message) Running Titanium compiler ...
	$(tc_command) -outdir $(generate_dir) $(ti_sources)
	touch $@

$(depends_mk) : $(makedepend) $(stamp_cache)
	$(message) Checking dependencies ...
	$< $(all_cc_flags) $(generate_dir)/*.c > $@
	echo "sources := " $(generate_dir)/*.c >> $@
	echo "headers := " $(generate_dir)/*.h >> $@

all : $(outfile)
ifeq "$(keep_intermediate)" "0"
	$(message) Removing intermediate files ...
	rm -f $(makefile) $(intermediate1)
	rm -f $(intermediate2)
	rm -f $(intermediate3)
	rm -f $(intermediate4)
	rm -f $(intermediate5)
	-rmdir $(generate_dir)
endif
	$(message) Done
